<?xml version="1.0" standalone="yes"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:junos="http://xml.juniper.net/junos/*/junos"
	xmlns:xnm="http://xml.juniper.net/xnm/1.1/xnm" xmlns:ext="http://xmlsoft.org/XSLT/namespace"
	xmlns:jcs="http://xml.juniper.net/junos/commit-scripts/1.0">

	<xsl:import href="../import/junos.xsl"/>

	<xsl:variable name="arguments">
		<argument>
			<name>username</name>
			<description>Username used to authenticate with the dyndns server</description>
		</argument>
		<argument>
			<name>password</name>
			<description>Password used to authenticate with the dyndns server</description>
		</argument>
		<argument>
			<name>type</name>
			<description>Dynamic dns server type</description>
		</argument>
		<argument>
			<name>url</name>
			<description>(optional) URL to use to update the IP</description>
		</argument>
		<argument>
			<name>hostname</name>
			<description>DNS hostname</description>
		</argument>
		<argument>
			<name>interface</name>
			<description>Interface to update</description>
		</argument>
		<argument>
			<name>force-ip</name>
			<description>IP address to use to update the host</description>
		</argument>
	</xsl:variable>
	<xsl:param name="username"/>
	<xsl:param name="password"/>
	<xsl:param name="type"/>
	<xsl:param name="url"/>
	<xsl:param name="interface"/>
	<xsl:param name="force-ip"/>

	<xsl:variable name="services">
		<service>
			<type>dyndns</type>
			<update-url>members.dyndns.org/nic/update?</update-url>
			<hostname-param>hostname</hostname-param>
			<ip-address-param>myip</ip-address-param>
			<sucess-message-regex>nochg|good</sucess-message-regex>
		</service>
		<service>
			<type>dtdns</type>
			<update-url>www.dtdns.com/api/autodns.cfm?</update-url>
			<hostname-param>id</hostname-param>
			<ip-address-param>myip</ip-address-param>
			<password-param>pw</password-param>
			<no-username/>
			<sucess-message-regex>nochg|good</sucess-message-regex>
		</service>
		<service>
			<type>dnspark</type>
			<update-url>www.dnspark.net/api/dynamic/update.php?mx=ON&amp;mxpri=5&amp;</update-url>
			<hostname-param>hostname</hostname-param>
			<sucess-message-regex>.*</sucess-message-regex>
		</service>
		<service>
			<type>3222</type>
			<update-url>http://www.3322.org/dyndns/update?system=dyndns&amp;</update-url>
			<hostname-param>hostname</hostname-param>
			<sucess-message-regex>.*</sucess-message-regex>
		</service>
		<service>
			<type>no-ip</type>
			<update-url>dynupdate.no-ip.com/nic/update?</update-url>
			<hostname-param>hostname</hostname-param>
			<ip-address-param>myip</ip-address-param>
			<sucess-message-regex>nochg|good</sucess-message-regex>
		</service>
		<service>
			<type>dns-o-matic</type>
			<update-url>updates.dnsomatic.com/nic/update?</update-url>
			<hostname-param>hostname</hostname-param>
			<sucess-message-regex>nochg|good</sucess-message-regex>
		</service>
		<service>
			<type>everydns</type>
			<update-url>dyn.everydns.net/index.php?ver=0.1&amp;</update-url>
			<hostname-param>domain</hostname-param>
			<sucess-message-regex>nochg|good</sucess-message-regex>
		</service>
		<service>
			<type>changeip</type>
			<update-url>nic.changeip.com/nic/update?system=dyndns&amp;</update-url>
			<hostname-param>hostname</hostname-param>
			<sucess-message-regex>nochg|good</sucess-message-regex>
		</service>
		<service>
			<type>freedns</type>
			<sucess-message-regex>has not changed</sucess-message-regex>
		</service>
		<service>
			<type>dnsexit</type>
			<update-url>BUOGUS:BOGUS@www.dnsexit.com/RemoteUpdate.sv?</update-url>
			<hostname-param>host</hostname-param>
			<username-param>login</username-param>
			<password-param>password</password-param>
			<no-username/>
			<sucess-message-regex>.*</sucess-message-regex>
		</service>
	</xsl:variable>

	<!-- Open a persistent connection -->
	<xsl:variable name="connection" select="jcs:open()"/>
	<xsl:variable name="get-dyndns-config">
		<rpc>
			<get-configuration>
				<configuration>
					<system>
						<services>
							<apply-macro/>
						</services>
					</system>
				</configuration>
			</get-configuration>
		</rpc>
	</xsl:variable>
	<xsl:variable name="dyndns-config" select="jcs:execute($connection, $get-dyndns-config)"/>

	<xsl:template name="update-server">
		<xsl:param name="url"/>
		<xsl:param name="retries"/>
		<xsl:param name="entry"/>
		<xsl:param name="sucess-message-regex"/>

		<xsl:if test="$retries &gt; 0">

			<xsl:variable name="clear-log-rpc">
				<rpc>
					<file-delete>
						<path>/var/log/dyndns.log</path>
					</file-delete>
				</rpc>
			</xsl:variable>
			<xsl:value-of select="jcs:execute($connection, $clear-log-rpc)"/>
			<xsl:variable name="update-rpc">
				<rpc>
					<file-copy>
						<source>
							<xsl:value-of select="$url"/>
						</source>
						<destination>/var/log/dyndns.log</destination>
					</file-copy>
				</rpc>
			</xsl:variable>
			<xsl:value-of select="jcs:execute($connection, $update-rpc)"/>

			<xsl:variable name="get-result-rpc">
				<rpc>
					<file-show>
						<filename>/var/log/dyndns.log</filename>
					</file-show>
				</rpc>
			</xsl:variable>
			<xsl:variable name="return" select="jcs:execute($connection, $get-result-rpc)"/>

			<!-- Check the result, if good or nochg we are done -->
			<xsl:variable name="result-parse" select="jcs:regex($sucess-message-regex,$return)"/>

			<xsl:choose>
				<xsl:when test="not($result-parse[1])">
					<!-- The update was not sucessfull, trying again -->
					<xsl:value-of
						select="jcs:syslog(12,concat('Unable to update dyndns entry ',$entry,
						' retries left:', $retries - 1, ' ', 
						' error:', $return))"/>
					<xsl:value-of select="jcs:sleep(1)"/>

					<xsl:call-template name="update-server">
						<xsl:with-param name="url" select="$url"/>
						<xsl:with-param name="retries" select="$retries - 1"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of
						select="jcs:syslog(12,concat('Dyndns server updated for ', $entry, ' status ', $return))"
					/>
				</xsl:otherwise>
			</xsl:choose>

		</xsl:if>
	</xsl:template>

	<xsl:template match="/">
		<op-script-results>

			<!-- Todo, only update the dyndns servers that are reachable throught the interface that has changed -->
			<xsl:for-each
				select="$dyndns-config//system/services/apply-macro[starts-with(name,'dyndns')]">
				<xsl:variable name="username" select="./data[name='username']/value"/>
				<xsl:variable name="password" select="./data[name='password']/value"/>
				<xsl:variable name="type" select="./data[name='type']/value"/>
				<xsl:variable name="hostname" select="./data[name='hostname']/value"/>
				<xsl:variable name="interface" select="./data[name='interface']/value"/>
				<xsl:variable name="force-ip" select="./data[name='force-ip']/value"/>
				<xsl:variable name="ip">
					<!-- We have to figure out if they entered an IP or an interface name -->
					<xsl:choose>
						<xsl:when test="$force-ip != ''">
							<xsl:value-of select="$force-ip"/>
						</xsl:when>
						<xsl:when test="$interface">
							<xsl:variable name="get-ip-rpc">
								<rpc>
									<get-interface-information>
										<interface-name>
											<xsl:value-of select="./data[name='interface']/value"/>
										</interface-name>
									</get-interface-information>
								</rpc>
							</xsl:variable>
							<xsl:variable name="get-ip"
								select="jcs:execute($connection, $get-ip-rpc)"/>
							<xsl:value-of
								select="ext:node-set($get-ip)//address-family[address-family-name='inet']/interface-address/ifa-local"
							/>
						</xsl:when>
					</xsl:choose>
				</xsl:variable>

				<!-- Construct the update URL based on the configured parameters -->
				<xsl:variable name="url">
					<xsl:choose>
						<xsl:when test="./data[name='url']">
							<xsl:value-of select="./data[name='url']/value"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>http://</xsl:text>
							<xsl:if
								test="not(ext:node-set($services)/service[type=$type]/no-username)">
								<xsl:value-of select="concat($username,':',$password,'@')"/>
							</xsl:if>
							<xsl:value-of
								select="concat(
								ext:node-set($services)/service[type=$type]/update-url,
								ext:node-set($services)/service[type=$type]/hostname-param, '=', $hostname)"
							/>
							<xsl:if test="$ip != '' and ext:node-set($services)/service[type=$type]/ip-address-param != ''">
								<xsl:value-of
									select="concat('&amp;',	ext:node-set($services)/service[type=$type]/ip-address-param, '=', $ip)"
									disable-output-escaping="yes"/>
							</xsl:if>
							<xsl:if test="ext:node-set($services)/service[type=$type]/username-param != ''">
								<xsl:value-of
									select="concat('&amp;',	ext:node-set($services)/service[type=$type]/username-param, '=', $username)"
									disable-output-escaping="yes"/>
							</xsl:if>
							<xsl:if test="ext:node-set($services)/service[type=$type]/password-param != ''">
								<xsl:value-of
									select="concat('&amp;',	ext:node-set($services)/service[type=$type]/password-param, '=', $password)"
									disable-output-escaping="yes"/>
							</xsl:if>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>

				<!-- Update the Dyndns server -->
				<xsl:call-template name="update-server">
					<xsl:with-param name="retries" select="2"/>
					<xsl:with-param name="url" select="$url"/>
					<xsl:with-param name="entry" select="./name"/>
					<xsl:with-param name="sucess-message-regex" select="ext:node-set($services)/service[type=$type]/sucess-message-regex"/>
				</xsl:call-template>

				<output>
					<xsl:copy-of select="$url"/>
				</output>

			</xsl:for-each>

		</op-script-results>
	</xsl:template>

</xsl:stylesheet>
